# -*- coding: utf-8 -*-

import os
import platform
import sys
from pathlib import Path

ROOT_DIR = os.path.dirname(os.path.abspath(__file__))

try:
	try:
		print("Attempting to load '._librapid' from {}".format(os.listdir(ROOT_DIR)))
		from ._librapid import *
	except ModuleNotFoundError:
		print("Failed to load '._librapid'. Attempting to load '_librapid' globally")
		from _librapid import *
except ImportError:
	print("Attempting to load librapid again, but using win32api to set the DLL directory")

	# If using windows, tell the os where the DLL for blas is
	# If blas was not installed, this doesn't really do anything
	if platform.system() == "Windows":
		import win32api
	
		if "openblas.dll" in os.listdir(ROOT_DIR) or \
			"libopenblas.dll" in os.listdir(ROOT_DIR):
			print("Loading DLL from './'")
			win32api.SetDllDirectory(ROOT_DIR)
			sys.path.append(ROOT_DIR)
		elif os.path.exists(os.path.join(ROOT_DIR, "blas")):
			print("Loading DLL from './blas'")
			win32api.SetDllDirectory(os.path.join(ROOT_DIR, "blas"))
			sys.path.append(os.path.join(ROOT_DIR, "blas"))

	try:
		try:
			print("Attempting to load '._librapid' from {}".format(os.listdir(ROOT_DIR)))
			from ._librapid import *
		except ModuleNotFoundError:
			print("Failed to load '._librapid'. Attempting to load '_librapid' globally")
			from _librapid import *
	except ImportError:
		print("There was an error trying to load the librapid C++ module '_librapid'")
		
		if platform.system() == "Windows":
			print("This could be caused by a missing BLAS DLL file in the '{}' directory".format(ROOT_DIR))
			print("\n".join(os.listdir(ROOT_DIR)))

		raise ImportError("Could not import '_librapid'")




# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.

import os
import sys
import textwrap

print("VERSION INFORMATION:")
os.system("pip show sphinx")
os.system("pip show breathe")
os.system("pip show exhale")
os.system("doxygen --version")

sys.path.insert(0, os.path.abspath("../.."))

# -- Project information -----------------------------------------------------

project = "librapid"
copyright = "2021, Toby Davis"
author = "Toby Davis"

# The full version, including alpha/beta/rc tags
version_file = open("../../src/librapid/VERSION.hpp", "r")
release = version_file.readlines()[1].split()[2].replace("\"", "")
version_file.close()

# Set the master file
master_doc = "index"

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named "sphinx.ext.*") or your custom
# ones.
extensions = [
	"furo",
	"sphinx.ext.autodoc",
	"sphinx.ext.napoleon",
	"sphinx.ext.coverage",
	"sphinx.ext.autosectionlabel",
	"sphinx.ext.intersphinx",
	"sphinx_panels",
	"breathe",
	"exhale",
	"numpydoc"
]

autosectionlabel_prefix_document = True

# Set up the breathe extension
breathe_projects = {
	"librapid": "./doxygenoutput/xml"
}

breathe_default_project = "librapid"

# Custom inputs to doxygen generator
doxygen_inputs = """

INPUT                 = ../../src/librapid
	   
ENABLE_PREPROCESSING  = YES
MACRO_EXPANSION       = YES
EXPAND_ONLY_PREDEF    = NO
PREDEFINED            += LR_INLINE=
PREDEFINED            += __restrict=
PREDEFINED            += LIBRAPID_MAX_DIMS=32
PREDEFINED            += __host__=
PREDEFINED            += __device__=
PREDEFINED            += __global__=

"""

# Set up the exhale extension
exhale_args = {
	# These arguments are required
	"containmentFolder":	"./api",
	"rootFileName":			"index.rst",
	"rootFileTitle":		"LibRapid",
	"doxygenStripFromPath":	"..",
	# Suggested optional arguments
	"createTreeView":		True,
	# TIP: if using the sphinx-bootstrap-theme, you need
	# "treeViewIsBootstrap": True,
	"exhaleExecutesDoxygen": True,
	"exhaleDoxygenStdin": textwrap.dedent(doxygen_inputs),
	"verboseBuild": False
}

# Add any paths that contain templates here, relative to this directory.
# templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.

# html_theme = "furo"
html_theme = "pydata_sphinx_theme"

html_logo = "../../branding/LibRapid small space.png"

html_context = {
    "github_user": "LibRapid",
    "github_repo": "librapid",
    "github_version": "master",
    "doc_path": "docs/source",
}

html_theme_options = {
    "logo_link": "https://librapid.readthedocs.io/en/latest/",
	"github_url": "https://github.com/LibRapid/librapid",
	"use_edit_page_button": True,
    "google_analytics_id": "G-LYHM6F499Q"
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ["_static"]



# -*- coding: utf-8 -*-
import os
from skbuild import setup
import distutils
import glob
from setuptools import find_packages
import platform
import shutil
from packaging.version import LegacyVersion
from skbuild.exceptions import SKBuildError
from skbuild.cmaker import get_cmake_version
import sys

# Copy dependencies
if os.path.exists("pybind11") and not os.path.exists(os.path.join("src", "librapid", "pybind11")):
	shutil.copytree("pybind11", os.path.join("src", "librapid", "pybind11"))

if os.path.exists("jitify") and not os.path.exists(os.path.join("src", "librapid", "jitify")):
	shutil.copytree("jitify", os.path.join("src", "librapid", "jitify"))

if os.path.exists("version2") and not os.path.exists(os.path.join("src", "librapid", "version2")):
	shutil.copytree("version2", os.path.join("src", "librapid", "version2"))

if os.path.exists("openblas_install") and not os.path.exists(os.path.join("src", "librapid", "openblas_install")):
	shutil.copytree("openblas_install", os.path.join("src", "librapid", "openblas_install"))

# if os.path.exists("_skbuild"):
# 	shutil.rmtree("_skbuild")

# Add CMake as a build requirement if cmake is not installed or is too low a version
setup_requires = []
install_requires = []

try:
	if LegacyVersion(get_cmake_version()) < LegacyVersion("3.10"):
		setup_requires.append('cmake')
		install_requires.append("cmake")
except SKBuildError:
	setup_requires.append('cmake')
	install_requires.append("cmake")

# ======= Uncomment this to install win32api as well =======
# if platform.system() == "Windows":
# 	setup_requires.append('pypiwin32')
# 	install_requires.append("pypiwin32")

ROOT_DIR = os.path.dirname(os.path.abspath(__file__))

print("\n\n\n\n")

data_files = []
if platform.system() == "Windows":
	for filename in ["openblas.dll", "libopenblas.dll"]:
		try:
			print("Attempting to open 'src/librapid/blas/bin{}".format(filename))
			with open(os.path.join("src", "librapid", "blas", "bin", filename), "r") as _:
				files = [os.path.join("src", "librapid", "blas", "bin", filename)]
				# data_files.append(("", files))
				data_files.append(("lib/site-packages/librapid", files))
				# data_files.append((distutils.sysconfig.get_python_lib(), files))
		except:
			print("Failed to open 'src/librapid/blas/bin/{}".format(filename))
			pass

	if data_files == []:
		for filename in ["openblas.dll", "libopenblas.dll"]:
			try:
				print("Attempting to open 'src/librapid/openblas_install/bin/{}".format(filename))
				with open(os.path.join("src", "librapid", "openblas_install", "bin", filename), "r") as _:
					files = [os.path.join("src", "librapid", "openblas_install", "bin", filename)]
					# data_files.append(("", files))
					data_files.append(("lib/site-packages/librapid", files))
					# data_files.append((distutils.sysconfig.get_python_lib(), files))
			except:
				print("Failed to open 'src/librapid/openblas_install/bin/{}".format(filename))
				pass

	if data_files == []:
		print("Was not able to load datafiles")
		print("File information:")
		if os.path.exists("src"):
			print("./src")
			print(os.listdir("./src"))

			if (os.path.exists("src/librapid")):
				print("./src/librapid")
				print(os.listdir("./src/librapid"))

				if (os.path.exists("src/librapid/openblas_install")):
					print("./src/librapid/openblas_install")
					print(os.listdir("./src/librapid/openblas_install"))

					if (os.path.exists("src/librapid/openblas_install/bin")):
						print("./src/librapid/openblas_install/bin")
						print(os.listdir("./src/librapid/openblas_install/bin"))

print("Operating System: {}".format(platform.system()))
print("Additional files being included: {}".format(data_files))
print("\n\n\n\n")

# Load the version number from VERSION.hpp
version_file = open(os.path.join("src", "librapid", "VERSION.hpp"), "r")
__version__ = version_file.readlines()[1].split()[2].replace("\"", "")
version_file.close()

# Locate and read the contents of README.md
with open(os.path.join(ROOT_DIR, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

if sys.maxsize > 2**32 and platform.system() == "Windows":
	print("Using 64bit Python")
	cmake_args = ["-DCMAKE_GENERATOR_PLATFORM=x64"]
else:
	print("Using 32bit Python (or unknown)")
	cmake_args = []

setup(
	name="librapid",
	version=__version__,
	author="Toby Davis",
	author_email="pencilcaseman@gmail.com",
	url="https://github.com/Pencilcaseman/librapid",
	description="A fast math and neural network library for Python and C++",
	long_description=long_description,
	long_description_content_type="text/markdown",
	packages=["librapid"],
	package_dir={"" : "src"},
	cmake_args=cmake_args,
	cmake_install_dir="src/librapid",
	license="MIT License",
	keywords=["math", "neural network", "ndarray", "array", "matrix",
			"high-performance computing"],
	classifiers=[
		"Development Status :: 2 - Pre-Alpha",
		"Intended Audience :: Developers",
		"Intended Audience :: Education",
		"License :: OSI Approved :: MIT License",
		"Programming Language :: Python",
		"Programming Language :: Python :: 3.6",
		"Programming Language :: Python :: 3.7",
		"Programming Language :: Python :: 3.8",
		"Programming Language :: Python :: 3.9",
	],
	extras_require={"test": "pytest"},
	install_requires=install_requires,
	setup_requires=setup_requires,
	data_files=data_files,
	include_package_data = True,
	zip_safe=False
)
